/*
 * EA872 - Lab. Programacao Software Basico
 *
 * Processamento do protocolo HTTP 1.1 com lex e yacc
 *
 * Eleri Cardozo, Marco A. A. Henriques, Ricardo R. Gudwin
 */


/*
 *  HTTP PDU format:
 *
 *  <request> /<resource> HTTP/1.1
 *  Host: <values>
 *  <parameter>: <values>
 *  ...
 *  <parameter>: <values>
 *  newline
 */


/*
 * Interaction with lex:
 *
 * yylex returns the type of token in a http pdu
 *
 */


%{
#include <stdio.h>
#include <list>
#include <vector>
#include <string.h>
#include <iostream>

char value[1024];
char resource[1024];
int varindex = 0;
int have_host = 0;   /* Host parameter is mandatory for HTTP 1.1 */

using namespace std;

list<string> *values = new list<string>;
vector<string> par_names;
vector< list<string> > par_values;
string last_par = "";

extern "C" {
    int yyparse(void);
    int yylex(void);
    int yywrap() { return 1; }
  }

void yyerror(const char *);
void store_value();
void store_last_parameter();
void print_resource();
void print_cab();

%}

%union {
   char text;
 }

%token NL CR COMMA END SLASH BLANK
%token GET HEAD OPTIONS TRACE POST PUT DELETE VERSION
%token VERSION11
%token CONNECTION USER_AGENT HOST ACCEPT ACCEPT_ENCODING ACCEPT_LANGUAGE ACCEPT_CHARSET KEEP_ALIVE IF_MODIFIED_SINCE CACHE_CONTROL REFERER TE AUTHORIZATION COOKIE PRAGMA
%token <text> CHARACTER

%%
http_message: request parameters;    /* what an HTTP message looks like */

request: reqtype resource version CR NL
       | reqtype resource version NL
       ;     /* the first line of an HTTP message */

reqtype: GET          {printf("Request: GET\n");}
       | HEAD         {}
       | OPTIONS      {}
       | TRACE        {} 
       | POST         {} 
       | PUT          {}
       | DELETE       {}
       | VERSION      {printf("Request: VERSION\n");}
       ;


resource: SLASH chars;    /* /arq.html */

chars: /* nothing */
     | chars char
     | chars slash char
     ;

slash: SLASH {value[varindex++] = '/';}
     ;

char: CHARACTER {value[varindex++] = yyval.text;}
    | BLANK {print_resource();}
    ;

version: VERSION11 {printf("\n\nVersion: 1.1");}
       ;

parameters: /* no parameter */
         | parameters parameter
         ;

parameter: partype fields   /* remaining parameters */
         ;

partype: CONNECTION        {store_last_parameter();
                            last_par = string("Connection");}
       | USER_AGENT        {}
       | ACCEPT            {}
       | ACCEPT_ENCODING   {}
       | ACCEPT_LANGUAGE   {}
       | ACCEPT_CHARSET    {}
       | KEEP_ALIVE        {}
       | IF_MODIFIED_SINCE {}
       | CACHE_CONTROL     {}
       | REFERER           {}
       | TE                {}
       | AUTHORIZATION     {}
       | COOKIE            {}
       | PRAGMA            {}
       | HOST              {have_host = 1;}
       ;


fields:  /* nothing */
      | fields field
      ;


field: CHARACTER                 /* a char belonging to a value */
               {if(! (varindex == 0 && yyval.text == ' '))
                  value[varindex++] = yyval.text;}
      | SLASH               /* slash - treat as character */
               {value[varindex++] = '/';}
      | BLANK               /* blank - treat as character */
               {value[varindex++] = ' ';}
      | COMMA               /* end of value */
               {store_value();}
      | NL                  /* end of value */
               {store_value();}
      | CR                  /* carriage return -- do nothing */
               {}
      | END                 /* end of parsing */
               {store_value();
               store_last_parameter();
	       print_cab();
               if(have_host == 0) yyerror("No HOST partameter");   /* host parameter absent */
               return(0);}
      | error {yyerror("Syntax error");}    /* if none above, report error */
      ;

%%


int lineno = 0;
extern char *yytext;


void print_cab() {
  // to be done
}


void store_last_parameter() {
  if(last_par.length() == 0) return;
  // cout << "\nStoring parameter " << last_par;
  if(values != NULL) {
    par_names.push_back(last_par);
    par_values.push_back(*values);
    // cout << "\nValues: ";
    // list<string>::iterator i;
    // for(i=values->begin(); i != values->end(); ++i) cout << *i << " ";
  }
  values = new list<string>;
}


void print_resource()
{
  int i,j=0;
  char *temp;

  temp = (char *) strdup(value);
  
  for(i = 0; i < varindex; i++) value[i + 1] = temp[j++];  /* shift */
  value[0] = '/';                  /* insert / */
  value[varindex+1] = '\0';             /* terminate string */
  varindex = 0;                       /* reset varindex */
  printf("\nResource: %s", value); /* print value */
  free(temp);
}

void store_value()
  {
    int i;
 
    if(varindex == 0) return;           /* no value */
    value[varindex] = '\0';             /* terminate string */
    varindex = 0;                       /* reset varindex */

    while(value[0]==' ') { /* shift */
      i=0;
      do {
	value[i]=value[i+1];
	i++;
      }
      while(value[i]!='\0');
    }
    values->push_back(string(value));
  }



void yyerror(const char *s)
 {
   printf("\nhttp parser error: %s", s);
   printf("\nline %d", lineno);
   printf("\nlast token read: |%s|\n", yytext);
 }
